@@ -2,8 +2,10 @@ from __future__ import print_function
 
 import six
 
+import copy
 from functools import partial
 import json
+import math
 import random
 import signal
 import time
@@ -97,7 +99,7 @@ class TaskQueue(ThreadedQueue):
     """
     return self._api.enqueued
     
-  def insert(self, task):
+  def insert(self, task, args=[], kwargs={}):
     """
     Insert a task into an existing queue.
     """
@@ -322,8 +324,8 @@ class MockTaskQueue(object):
   def __init__(self, *args, **kwargs):
     pass
 
-  def insert(self, task):
-    task.execute()
+  def insert(self, task, args=[], kwargs={}):
+    task.execute(*args, **kwargs)
     del task
 
   def poll(self, *args, **kwargs):
@@ -350,8 +352,8 @@ class LocalTaskQueue(object):
     self.queue = []
     self.progress = progress
 
-  def insert(self, task):
-    self.queue.append(task)
+  def insert(self, task, args=[], kwargs={}):
+    self.queue.append( (task, args, kwargs) )
 
   def wait(self, progress=None):
     return self
@@ -368,11 +370,52 @@ class LocalTaskQueue(object):
   def __exit__(self, exception_type, exception_value, traceback):
     with tqdm(total=len(self.queue), desc="Tasks") as pbar:
       with concurrent.futures.ProcessPoolExecutor(max_workers=self.parallel) as executor:
-        for _ in executor.map(task_execute, self.queue):
+        for _ in executor.map(_task_execute, self.queue):
           pbar.update()
     self.queue = []
 
 # Necessary to define here to make the 
 # function picklable
-def task_execute(task):
-  task.execute()
+def _task_execute(task_tuple):
+  task, args, kwargs = task_tuple
+  task.execute(*args, **kwargs)
+
+## Multiprocess Upload
+
+def _scatter(sequence, n):
+  """Scatters elements of ``sequence`` into ``n`` blocks."""
+
+  output = []
+  chunklen = int(math.ceil(float(len(sequence)) / float(n)))
+
+  for i in range(n):
+    output.append(
+      copy.deepcopy(sequence[i*chunklen:(i+1)*chunklen])
+    )
+
+  return output
+
+def _upload_tasks_single_process(queue_name, tasks):
+  with TaskQueue(queue_name=queue_name, queue_server='sqs') as tq:
+    for task in tasks:
+      tq.insert(task)
+
+def upload(queue_name, tasks, parallel=16, progress=False):
+  if parallel == 0:
+    parallel = mp.cpu_count()
+  elif parallel < 0:
+    raise ValueError("Parallel must be a positive number or zero (all cpus). Got: " + str(parallel))
+
+  parallel = max(1, min(parallel, mp.cpu_count()))
+
+  if parallel == 1:
+    _upload_tasks_single_process(queue_name, tasks)
+  else:
+    uploadfn = partial(_upload_tasks_single_process, queue_name)
+    tasks = _scatter(tasks, parallel)
+
+    with concurrent.futures.ProcessPoolExecutor(max_workers=parallel) as executor:
+      executor.map(uploadfn, tasks)
+
+
+
