@@ -5,6 +5,7 @@ import time
 from six.moves import range
 import pytest
 
+import taskqueue
 from taskqueue import RegisteredTask, TaskQueue, MockTask, PrintTask, LocalTaskQueue
 from taskqueue import QUEUE_NAME
 
@@ -24,9 +25,19 @@ QTYPES = ('aws',)#, 'google')
 
 QURL = 'https://sqs.us-east-1.amazonaws.com/098703261575/wms-test-pull-queue'
 
+class ExecutePrintTask(RegisteredTask):
+  def __init__(self):
+    super().__init__()
+
+  def execute(self, wow, wow2):
+    print(wow + wow2)
+    return wow + wow2
+
+
 def test_task_creation():
   task = MockTask(this="is", a=[1, 4, 2], simple={"test": "to", "check": 4},
-               serialization=('i', 's', 's', 'u', 'e', 's'), with_kwargs=None)
+               serialization=('i', 's', 's', 'u', 'e', 's'), wow=4, with_kwargs=None)
+  task.wow = 5
   payload = task.payload()
   payload = json.dumps(payload)
   task_deserialized = MockTask.deserialize(payload)
@@ -35,6 +46,7 @@ def test_task_creation():
       "this": "is", "a": [1, 4, 2],
       "simple": {"test": "to", "check": 4},
       "serialization": ["i", "s", "s", "u", "e", "s"],
+      "wow": 5,
       "with_kwargs": None
   }
 
@@ -107,6 +119,22 @@ def test_multi_threaded_insertion():
     if qtype != 'aws':
       assert tq.enqueued == 0
 
+def test_multiprocess_upload():
+  global QURL
+
+  num_tasks = 1000
+  tasks = [ PrintTask(i) for i in range(num_tasks) ]
+
+  taskqueue.upload(QURL, tasks, parallel=4)
+
+  tq = TaskQueue(QURL, n_threads=0)
+
+  time.sleep(0.25)
+  try:
+    assert tq.enqueued == num_tasks
+  finally:
+    tq.purge()
+
 def test_400_errors():
   global QUEUE_NAME
   for qtype in QTYPES:
@@ -118,4 +146,8 @@ def test_local_taskqueue():
     for i in range(20000):
       tq.insert(
         MockTask(arg=i)
-      )
\ No newline at end of file
+      )
+
+  with LocalTaskQueue(parallel=True, progress=False) as tq:
+    for i in range(200):
+      tq.insert(ExecutePrintTask(), [i], { 'wow2': 4})
